@@ -1331,6 +1331,7 @@ g_vfs_backend_google_delete (GVfsBackend   *_self,
   gchar *entry_path = NULL;
   GList *parent_ids;
   guint parent_ids_len;
+  gchar *id = NULL;
 
   g_rec_mutex_lock (&self->mutex);
   g_debug ("+ delete: %s\n", filename);
@@ -1344,6 +1345,39 @@ g_vfs_backend_google_delete (GVfsBackend   *_self,
       goto out;
     }
 
+  /* g_strdup() is necessary to prevent segfault because gdata_entry_get_id() calls g_free() */
+  id = g_strdup (gdata_entry_get_id (entry));
+
+  if (GDATA_IS_DOCUMENTS_FOLDER (entry))
+    {
+      GHashTableIter iter;
+      DirEntriesKey *key;
+
+      if (!is_dir_listing_valid (self, entry))
+        {
+          rebuild_dir (self, entry, cancellable, &error);
+          if (error != NULL)
+            {
+              g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
+              g_error_free (error);
+              goto out;
+            }
+        }
+
+      g_hash_table_iter_init (&iter, self->dir_entries);
+      while (g_hash_table_iter_next (&iter, (gpointer *) &key, NULL))
+        {
+          if (g_strcmp0 (key->parent_id, id) == 0)
+            {
+              g_vfs_job_failed (G_VFS_JOB (job),
+                                G_IO_ERROR,
+                                G_IO_ERROR_NOT_EMPTY,
+                                _("Directory not empty"));
+              goto out;
+            }
+        }
+    }
+
   parent = resolve_dir (self, filename, cancellable, NULL, NULL, &error);
   if (error != NULL)
     {
@@ -1392,6 +1426,7 @@ g_vfs_backend_google_delete (GVfsBackend   *_self,
       sanitize_error (&error);
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
       g_error_free (error);
+      g_object_unref (entry);
       goto out;
     }
 
@@ -1403,11 +1438,12 @@ g_vfs_backend_google_delete (GVfsBackend   *_self,
     insert_entry (self, GDATA_ENTRY (new_entry));
   g_hash_table_foreach (self->monitors, emit_delete_event, entry_path);
   g_vfs_job_succeeded (G_VFS_JOB (job));
+  g_object_unref (entry);
 
  out:
-  g_object_unref (entry);
   g_clear_object (&new_entry);
   g_free (entry_path);
+  g_free (id);
   g_debug ("- delete\n");
   g_rec_mutex_unlock (&self->mutex);
 }
@@ -2410,6 +2446,11 @@ g_vfs_backend_google_set_display_name (GVfsBackend           *_self,
       goto out;
     }
 
+  /* The internal ref count has to be increased before removing the entry since
+   * remove_entry_full calls g_object_unref(). */
+  g_object_ref (entry);
+  remove_entry (self, entry);
+
   gdata_entry_set_title (entry, display_name);
   auth_domain = gdata_documents_service_get_primary_authorization_domain ();
 
@@ -2420,14 +2461,15 @@ g_vfs_backend_google_set_display_name (GVfsBackend           *_self,
       sanitize_error (&error);
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
       g_error_free (error);
+      g_object_unref (entry);
       goto out;
     }
 
-  remove_entry (self, entry);
   insert_entry (self, new_entry);
   g_hash_table_foreach (self->monitors, emit_attribute_changed_event, entry_path);
   g_vfs_job_set_display_name_set_new_path (job, entry_path);
   g_vfs_job_succeeded (G_VFS_JOB (job));
+  g_object_unref (entry);
 
  out:
   g_clear_object (&new_entry);
